# coding: utf8
u"""Экшены и паки приложения логирования СМЭВ."""
import datetime
import functools

from django.db.models import get_model, Q

import objectpack

from educommon.utils.ui import ChoicesFilter, ColumnFilterWithDefaultValue

from . import models
from . import ui


ALL_TYPES = -1  # Вид взаимодействия - Все


class SmevLogPack(objectpack.ObjectPack):
    u"""Лог запросов СМЭВ."""

    model = models.SmevLog
    edit_window = ui.SmevLogEditWindow
    list_window = ui.SmevLogListWindow

    column_name_on_select = 'method_name'
    list_sort_order = ['-time']
    can_delete = False

    filter_engine_clz = objectpack.filters.ColumnFilterEngine
    filter_field = functools.partial(objectpack.filters.FilterByField, model)
    date_field = functools.partial(ColumnFilterWithDefaultValue, model)

    columns = [
        {
            'data_index': 'service_address',
            'header': u'Адрес сервиса',
            'filter': filter_field(
                'service_address', 'service_address__icontains')
        },
        {
            'data_index': 'method_name',
            'header': u'Код метода',
            'filter': filter_field('method_name', 'method_name__icontains')
        },
        {
            'data_index': 'method_verbose_name',
            'header': u'Наименование метода',
            'filter': filter_field(
                'method_verbose_name', 'method_verbose_name__icontains')
        },
        {
            'data_index': 'time',
            'header': u'Дата время',
            'filter': date_field(
                'time',
                lookup=lambda d: Q(
                    time__year=d.year, time__month=d.month, time__day=d.day
                ) if d else Q(),
                tooltip=u'Дата время',
                value=datetime.date.today(),
                allow_blank=False
            )
        },
        {
            'data_index': 'error',
            'header': u'Ошибка',
        },
        {
            'data_index': 'interaction_type_verbose',
            'header': u'Вид взаимодействия',
            'filter': ChoicesFilter(
                choices=models.SmevLog.INTERACTION_TYPES + (
                    (ALL_TYPES, u'Все'),),
                parser=int,
                lookup=lambda index: Q(
                    interaction_type=index) if index != ALL_TYPES else Q(),
                tooltip=u'Вид взаимодействия'
            )
        }
    ]

    # Название фильтра для столбца "Дата время" в ajax-запросе
    date_time_filter_param_name = 'filter_4'

    def configure_grid(self, grid):
        u"""Настройка грида."""
        super(SmevLogPack, self).configure_grid(grid)
        grid.top_bar.button_edit.text = u'Просмотр'

        grid.store.base_params = {
            self.date_time_filter_param_name: str(datetime.date.today())
        }

    def prepare_row(self, obj, request, context):
        u"""Настройка строки грида, вызывается посточно для каждой строки."""
        obj.interaction_type_verbose = (
            dict(self.model.INTERACTION_TYPES)[obj.interaction_type])

        return obj

    def get_edit_window_params(self, params, request, context):
        u"""Дополняет параметры для окна редактирования."""
        log = params['object']
        method_name = log.method_name or log.method_verbose_name

        params['title'] = u'Лог по методу: {}'.format(method_name)

        return params

    def extend_menu(self, menu):
        u"""Размещение в меню."""
        return menu.SubMenu(
            u'Администрирование',
            menu.SubMenu(
                u'Взаимодействие со СМЭВ',
                menu.Item(
                    u'Логи СМЭВ',
                    pack=self.get_default_action()
                )
            )
        )


class SmevProviderPack(objectpack.ObjectPack):

    u"""Пак поставщики СМЭВ."""

    model = models.SmevProvider
    add_window = edit_window = ui.SmevProviderEditWindow
    list_window = ui.SmevProviderListWindow

    columns = [
        {
            'data_index': 'mnemonics',
            'header': u'Мнемоника',
            'sortable': True,
            'searchable': True,
            'width': 1
        },
        {
            'data_index': 'address',
            'header': u'Адрес СМЭВ',
            'sortable': True,
            'searchable': True,
            'width': 1
        },
        {
            'data_index': 'service_name',
            'header': u'Наименование электронного сервиса',
            'sortable': True,
            'searchable': True,
            'width': 3
        },
        {
            'data_index': 'source',
            'header': u'Источник взаимодействия',
            'sortable': True,
            'searchable': True,
            'width': 2
        },
        {
            'data_index': 'entity',
            'header': u'Наименование юр. лица',
            'sortable': True,
            'searchable': True,
            'width': 2
        },
        {
            'data_index': 'service_address_status_changes',
            'header': u'Адрес сервиса изменения статуса',
            'sortable': True,
            'searchable': True,
            'width': 3
        }
    ]

    def extend_menu(self, menu):
        u"""Размещение в меню."""
        return menu.SubMenu(
            u'Администрирование',
            menu.SubMenu(
                u'Взаимодействие со СМЭВ',
                menu.Item(
                    u'Поставщики СМЭВ',
                    pack=self.get_default_action()
                )
            )
        )
